package cn.com.qmth.stmms.biz.config.service.impl;

import javax.annotation.PostConstruct;

import net.sf.json.JSONObject;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Component;

import cn.com.qmth.stmms.biz.config.model.SystemAuth;
import cn.com.qmth.stmms.biz.config.service.SystemAuthService;
import cn.com.qmth.stmms.biz.config.service.SystemConfigService;
import cn.com.qmth.stmms.biz.utils.AppLicenseUtil;
import cn.com.qmth.stmms.biz.utils.SolarHttpUtil;
import cn.com.qmth.stmms.common.enums.ConfigType;
import cn.com.qmth.stmms.common.enums.SystemAuthType;
import cn.com.qmth.stmms.common.utils.AesUtils;

import com.qmth.boot.tools.models.ByteArray;

@Component
public class SystemCache {

    @Autowired
    private SystemAuthService authService;

    @Autowired
    private SystemConfigService configService;

    private boolean Auth;

    private Long expireTime;

    private String fileServer;

    private Long markTime;

    @Value("${qmth.solar.host}")
    private String host;

    @Value("${qmth.solar.app.uri}")
    private String uri;

    @PostConstruct
    public void init() {
        SystemAuth s = authService.findOne();
        if (s != null && SystemAuthType.ONLINE.equals(s.getType())) {
            String accessKey = AesUtils.decrypt(s.getAccessKey());
            String accessSecret = AesUtils.decrypt(s.getAccessSecret());
            SolarHttpUtil httpUtil = new SolarHttpUtil(accessKey, accessSecret, host, uri);
            if (httpUtil.getAPP()) {
                Auth = true;
            }
        } else if (s != null && SystemAuthType.OFFLINE.equals(s.getType())) {
            JSONObject json = AppLicenseUtil.parseLicense(ByteArray.fromBase64(s.getDescription()).value());
            if (json != null) {
                Auth = true;
                if (json.has("control") && json.getJSONObject("control").has("expireTime")
                        && !json.getJSONObject("control").getJSONObject("expireTime").isNullObject()) {
                    expireTime = json.getJSONObject("control").getLong("expireTime");
                }
            }
        } else {
            Auth = false;
        }
        fileServer = configService.findByType(ConfigType.FILE_SERVER);
        markTime = Long.parseLong(configService.findByType(ConfigType.MARK_TIME));
    }

    public boolean isAuth() {
        return Auth;
    }

    public void setAuth(boolean auth) {
        Auth = auth;
    }

    public String getFileServer() {
        return fileServer;
    }

    public void setFileServer(String fileServer) {
        this.fileServer = fileServer;
    }

    public Long getMarkTime() {
        return markTime;
    }

    public void setMarkTime(Long markTime) {
        this.markTime = markTime;
    }

    public Long getExpireTime() {
        return expireTime;
    }

    public void setExpireTime(Long expireTime) {
        this.expireTime = expireTime;
    }

}
