package cn.com.qmth.stmms.admin.exam;

import cn.com.qmth.stmms.admin.dto.RejectResult;
import cn.com.qmth.stmms.admin.vo.InspectedSubjectVO;
import cn.com.qmth.stmms.biz.exam.model.Exam;
import cn.com.qmth.stmms.biz.exam.model.ExamStudent;
import cn.com.qmth.stmms.biz.exam.model.ExamSubject;
import cn.com.qmth.stmms.biz.exam.query.ExamStudentSearchQuery;
import cn.com.qmth.stmms.biz.exam.service.*;
import cn.com.qmth.stmms.biz.exam.service.query.ExamSubjectSearchQuery;
import cn.com.qmth.stmms.biz.file.service.FileService;
import cn.com.qmth.stmms.biz.lock.LockService;
import cn.com.qmth.stmms.biz.mark.model.Task;
import cn.com.qmth.stmms.biz.mark.service.MarkService;
import cn.com.qmth.stmms.biz.mark.service.TaskService;
import cn.com.qmth.stmms.common.annotation.Logging;
import cn.com.qmth.stmms.common.annotation.RoleRequire;
import cn.com.qmth.stmms.common.domain.WebUser;
import cn.com.qmth.stmms.common.enums.*;
import cn.com.qmth.stmms.common.utils.RequestUtils;
import net.sf.json.JSONObject;
import org.apache.commons.lang.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Controller;
import org.springframework.ui.Model;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.servlet.mvc.support.RedirectAttributes;

import javax.servlet.http.HttpServletRequest;
import java.text.DecimalFormat;
import java.util.ArrayList;
import java.util.LinkedList;
import java.util.List;
import java.util.Set;

@Controller("inspectedController")
@RequestMapping("/admin/exam/inspected")
public class InspectedController extends BaseExamController {

    protected static Logger log = LoggerFactory.getLogger(InspectedController.class);

    @Autowired
    private ExamStudentService studentService;

    @Autowired
    private InspectedService inspectedService;

    @Autowired
    private ExamQuestionService questionService;

    @Autowired
    private FileService fileService;

    @Autowired
    private LockService lockService;

    @Autowired
    private MarkService markService;

    @Autowired
    private ExamService examService;

    @Autowired
    private TaskService taskService;

    @Autowired
    private ExamSubjectService subjectService;

    @Autowired
    private InspectHistoryService inspectHistoryService;

    @Value("${slice.split.config}")
    private String splitConfig;

    public static final String UN_SELECTIVE_SCORE = "-1";

    @RequestMapping(value = "/list")
    public String list(Model model, HttpServletRequest request, ExamStudentSearchQuery query,
            @RequestParam(required = false) SubjectiveStatus status,
            @RequestParam(required = false) SelectiveStatus selectiveStatus,
            @RequestParam(required = false) Integer mainNumber, @RequestParam(required = false) Double mainStartScore,
            @RequestParam(required = false) Double mainEndScore, @RequestParam(required = false) Double questionScore) {
        int examId = getSessionExamId(request);
        Exam exam = examService.findById(examId);
        if (ExamType.MULTI_MEDIA.equals(exam.getType())) {
            return "modules/exam/permission";
        }
        WebUser wu = RequestUtils.getWebUser(request);
        query.setExamId(examId);
        List<ExamSubject> subjectList = getExamSubject(examId, wu);
        if (StringUtils.isBlank(query.getSubjectCode()) && wu.getRole() != Role.SCHOOL_ADMIN) {
            List<String> subjectCodeIn = new ArrayList<String>();
            subjectList.forEach(e -> subjectCodeIn.add(e.getCode()));
            query.setSubjectCodeIn(StringUtils.join(subjectCodeIn, ","));
        }
        List<ExamStudent> list = inspectedService
                .findByQuery(query, status, mainNumber, mainStartScore, mainEndScore, questionScore, selectiveStatus);
        for (ExamStudent e : list) {
            e.setSubjectiveScoreList(e.getSubjectiveScoreList().replace(UN_SELECTIVE_SCORE, "/"));
            e.setInspectHistoryList(inspectHistoryService.findByStudentIdLimit3(e.getId()));
        }
        Integer totalCount = inspectedService
                .countByQuery(query, status, mainNumber, mainStartScore, mainEndScore, questionScore, selectiveStatus);
        Integer inspectCount = inspectedService
                .countByQuery(query, SubjectiveStatus.MARKED, mainNumber, mainStartScore, mainEndScore, questionScore,
                        selectiveStatus);
        query.setResult(list);
        query.setTotalCount(totalCount);
        model.addAttribute("query", query);
        model.addAttribute("inspectCount", inspectCount);
        model.addAttribute("questionList",
                questionService.findMainByExamAndSubjectAndObjective(examId, query.getSubjectCode(), false));
        model.addAttribute("mainNumber", mainNumber);
        model.addAttribute("mainStartScore", mainStartScore);
        model.addAttribute("mainEndScore", mainEndScore);
        model.addAttribute("questionScore", questionScore);
        model.addAttribute("subjectList", subjectList);
        model.addAttribute("statusList", SubjectiveStatus.getOptionList());
        model.addAttribute("selectiveStatusList", SelectiveStatus.getOptionList());
        model.addAttribute("status", status);
        model.addAttribute("selectiveStatus", selectiveStatus);
        model.addAttribute("exam", exam);
        return "modules/exam/inspectedList";
    }

    @Logging(menu = "开始考生复核", type = LogType.QUERY)
    @RequestMapping(value = "/getTask", method = RequestMethod.POST)
    @ResponseBody
    public Task getTask(HttpServletRequest request, RedirectAttributes redirectAttributes, ExamStudentSearchQuery query,
            @RequestParam(required = false, defaultValue = "false") SelectiveStatus selectiveStatus,
            @RequestParam(required = false) Integer mainNumber, @RequestParam(required = false) Double mainStartScore,
            @RequestParam(required = false) Double mainEndScore, @RequestParam(required = false) Double questionScore,
            @RequestParam(required = false) Integer studentId) {
        int examId = getSessionExamId(request);
        WebUser wu = RequestUtils.getWebUser(request);
        Task task = null;
        if (studentId != null) {
            releaseStudent(studentId);
            ExamStudent student = studentService.findById(studentId);
            if (inspectedService.applyStudent(student, wu.getId())) {
                task = taskService.build(student);
            }
        }
        if (StringUtils.isNotBlank(query.getSecretNumber())) {
            ExamStudent student = studentService.findByExamIdAndSecretNumber(examId, query.getSecretNumber());
            if (student == null) {
                return null;
            }
            releaseStudent(student.getId());
            if (inspectedService.applyStudent(student, wu.getId())) {
                task = taskService.build(student);
            }
        }
        int retry = 1;
        while (task == null) {
            List<ExamStudent> list = new ArrayList<ExamStudent>();
            // 需要判断评卷员是否绑定了班级
            query.setExamId(examId);
            query.setPageNumber(retry);
            query.setPageSize(20);
            List<ExamSubject> subjectList = getExamSubject(examId, wu);
            if (StringUtils.isBlank(query.getSubjectCode()) && !subjectList.isEmpty()) {
                query.setSubjectCode(subjectList.get(0).getCode());
            }
            list = inspectedService
                    .findByQuery(query, SubjectiveStatus.MARKED, mainNumber, mainStartScore, mainEndScore,
                            questionScore, selectiveStatus);
            if (list.isEmpty()) {
                break;
            }
            for (ExamStudent student : list) {
                if (inspectedService.applyStudent(student, wu.getId())) {
                    task = taskService.build(student);
                    break;
                }
            }
            if (task == null) {
                retry++;
            }
        }
        return task;
    }

    @Logging(menu = "考生评卷复核", type = LogType.UPDATE)
    @RequestMapping(value = "/save", method = RequestMethod.POST)
    @ResponseBody
    public Object save(HttpServletRequest request, @RequestParam Integer studentId) {
        WebUser wu = RequestUtils.getWebUser(request);
        ExamStudent student = studentService.findById(studentId);
        JSONObject obj = new JSONObject();
        try {
            if (student != null && lockService.trylock(LockType.STUDENT, studentId) && inspectedService
                    .inspect(student, wu.getId())) {
                inspectedService.releaseByStudent(student);
                obj.accumulate("success", true);
            } else {
                obj.accumulate("success", false);
                obj.accumulate("message", "无法复核，请刷新页面");
            }
        } catch (Exception e) {
            obj.accumulate("success", false);
            log.error("inspected save error", e);
        } finally {
            lockService.unlock(LockType.STUDENT, studentId);
        }
        return obj;
    }

    @RequestMapping(value = "/clear", method = RequestMethod.POST)
    @ResponseBody
    @RoleRequire({ Role.SCHOOL_ADMIN, Role.SUBJECT_HEADER, Role.INSPECTOR, Role.COLLEGE_ADMIN })
    public JSONObject clear(HttpServletRequest request, @RequestParam(required = false) String subjectCode,
            @RequestParam(required = false) Integer studentId) {
        JSONObject obj = new JSONObject();
        if (studentId != null) {
            releaseStudent(studentId);
            obj.accumulate("success", true);
        } else if (subjectCode != null) {
            int examId = getSessionExamId(request);
            releaseUser(examId, subjectCode, RequestUtils.getWebUser(request).getId());
            obj.accumulate("success", true);
        } else {
            obj.accumulate("success", false);
        }
        return obj;
    }

    @Logging(menu = "取消复核", type = LogType.UPDATE)
    @RequestMapping(value = "/cancel", method = RequestMethod.POST)
    @ResponseBody
    @RoleRequire({ Role.SCHOOL_ADMIN, Role.SUBJECT_HEADER, Role.INSPECTOR, Role.COLLEGE_ADMIN })
    public JSONObject cancel(HttpServletRequest request, @RequestParam Integer studentId) {
        JSONObject obj = new JSONObject();
        ExamStudent student = studentService.findById(studentId);
        obj.accumulate("subjectCode", student.getSubjectCode());
        WebUser wu = RequestUtils.getWebUser(request);
        if (student != null && subjectCheck(student.getSubjectCode(), wu)) {
            try {
                if (lockService.trylock(LockType.STUDENT, studentId) && inspectedService.cancel(studentId)) {
                    obj.accumulate("success", true);
                } else {
                    obj.accumulate("success", false);
                    obj.accumulate("message", "无法取消复核");
                }
            } catch (Exception e) {
                obj.accumulate("success", false);
                obj.accumulate("message", "取消复核失败");
                log.error("back inspected error", e);
            } finally {
                lockService.unlock(LockType.STUDENT, studentId);
            }
        } else {
            obj.accumulate("success", false);
            obj.accumulate("message", "没有操作该评卷任务的权限");
        }
        return obj;
    }

    @Logging(menu = "打回", type = LogType.UPDATE)
    @RequestMapping(value = "/rejected", method = RequestMethod.POST)
    @ResponseBody
    @RoleRequire({ Role.SCHOOL_ADMIN, Role.SUBJECT_HEADER, Role.INSPECTOR, Role.COLLEGE_ADMIN })
    public JSONObject rejected(HttpServletRequest request, @RequestBody RejectResult rejectResult) {
        JSONObject obj = new JSONObject();
        ExamStudent student = studentService.findById(rejectResult.getStudentId());
        WebUser wu = RequestUtils.getWebUser(request);
        if (student == null) {
            obj.accumulate("success", false);
            obj.accumulate("message", "无法打回");
            return obj;
        }
        if ((inspectedService.hasApplied(student, wu.getId()) && (student.getSubjectiveStatus()
                .equals(SubjectiveStatus.MARKED))) || student.getSubjectiveStatus()
                .equals(SubjectiveStatus.INSPECTED)) {
            if (subjectCheck(student.getSubjectCode(), wu) && lockService.trylock(LockType.STUDENT, student.getId())) {
                try {
                    lockService.watch(LockType.EXAM_SUBJECT, student.getExamId(), student.getSubjectCode());
                    if (markService.rejectedStudent(student, rejectResult.getQuestionList(), wu.getId(),
                            rejectResult.getReason())) {
                        inspectedService.releaseByStudent(student);
                        obj.accumulate("success", true);
                    } else {
                        obj.accumulate("success", false);
                        obj.accumulate("message", "无法打回");
                    }
                } catch (Exception e) {
                    obj.accumulate("success", false);
                    obj.accumulate("message", "打回失败");
                    log.error("back inspected error", e);
                } finally {
                    lockService.unwatch(LockType.EXAM_SUBJECT, student.getExamId(), student.getSubjectCode());
                    lockService.unlock(LockType.STUDENT, student.getId());
                }
            } else {
                obj.accumulate("success", false);
                obj.accumulate("message", "无法打回");
            }
        }
        return obj;
    }

    @Logging(menu = "回看全卷复核任务", type = LogType.QUERY)
    @RequestMapping(value = "/getHistory", method = RequestMethod.POST)
    @ResponseBody
    public Object getHistory(HttpServletRequest request, @RequestParam String subjectCode, @RequestParam int pageNumber,
            @RequestParam int pageSize) throws Exception {
        int examId = getSessionExamId(request);
        WebUser wu = RequestUtils.getWebUser(request);
        List<Task> list = new ArrayList<>();
        ExamStudentSearchQuery query = new ExamStudentSearchQuery();
        query.setExamId(examId);
        query.setSubjectCode(subjectCode);
        query.setInspectorId(wu.getId());
        query.addStatus(SubjectiveStatus.INSPECTED);
        query.setPageNumber(pageNumber);
        query.setPageSize(pageSize);
        query.orderByInspectTimeDesc();
        query = studentService.findByQuery(query);
        for (ExamStudent student : query.getResult()) {
            Task task = taskService.build(student);
            task.setPrevious(true);
            list.add(task);
        }
        return list;
    }

    @RequestMapping(value = "/getStatus", method = RequestMethod.POST)
    @ResponseBody
    public JSONObject status(HttpServletRequest request, ExamStudentSearchQuery query,
            @RequestParam(required = false) SelectiveStatus selectiveStatus,
            @RequestParam(required = false) Integer mainNumber, @RequestParam(required = false) Double mainStartScore,
            @RequestParam(required = false) Double mainEndScore, @RequestParam(required = false) Double questionScore) {
        JSONObject status = new JSONObject();
        int examId = getSessionExamId(request);
        query.setExamId(examId);
        Integer totalCount = inspectedService
                .countByQuery(query, SubjectiveStatus.MARKED, mainNumber, mainStartScore, mainEndScore, questionScore,
                        selectiveStatus);
        status.accumulate("totalCount", totalCount);
        status.accumulate("valid", true);
        return status;
    }

    @RequestMapping(value = "/getSetting", method = RequestMethod.POST)
    @ResponseBody
    public JSONObject getSetting(HttpServletRequest request, @RequestParam String subjectCode) {
        JSONObject setting = new JSONObject();
        WebUser wu = RequestUtils.getWebUser(request);
        int examId = getSessionExamId(request);
        Exam exam = examService.findById(examId);
        setting.accumulate("examType", exam.getType());
        setting.accumulate("fileServer", fileService.getFileServer());
        setting.accumulate("userName", wu.getName());
        setting.accumulate("splitConfig", getSplitConfig());

        ExamSubject examSubject = subjectService.find(examId, subjectCode);
        JSONObject subject = new JSONObject();
        subject.accumulate("name", examSubject.getName());
        subject.accumulate("code", examSubject.getCode());
        examSubject.setPaperAnswerUrl(fileService);
        subject.accumulate("paperUrl", examSubject.getPaperUrl() == null ? "" : examSubject.getPaperUrl());
        subject.accumulate("answerUrl", examSubject.getAnswerUrl() == null ? "" : examSubject.getAnswerUrl());
        setting.accumulate("subject", subject);
        setting.accumulate("enableSplit",
                examSubject.getEnableSplit() == null ? exam.isEnableSplit() : examSubject.getEnableSplit());
        return setting;
    }

    private double[] getSplitConfig() {
        String strs[] = splitConfig.split(",");
        double[] config = new double[strs.length];
        for (int i = 0; i < strs.length; i++) {
            config[i] = Double.parseDouble(strs[i]);
        }
        return config;
    }

    private void releaseUser(Integer examId, String subjectCode, Integer userId) {
        try {
            lockService.waitlock(LockType.USER, userId);
            inspectedService.releaseByUserId(examId, subjectCode, userId);
        } catch (Exception e) {
            log.error("release user error", e);
        } finally {
            lockService.unlock(LockType.USER, userId);
        }
    }

    private void releaseStudent(Integer studentId) {
        try {
            lockService.waitlock(LockType.STUDENT, studentId);
            ExamStudent student = studentService.findById(studentId);
            inspectedService.releaseByStudent(student);
        } catch (Exception e) {
            log.error("release user error", e);
        } finally {
            lockService.unlock(LockType.STUDENT, studentId);
        }
    }

    @Logging(menu = "取消复核", type = LogType.UPDATE)
    @RequestMapping(value = "/batchCancel", method = RequestMethod.POST)
    @RoleRequire({ Role.SCHOOL_ADMIN, Role.SUBJECT_HEADER, Role.INSPECTOR, Role.COLLEGE_ADMIN })
    public String batchCancel(HttpServletRequest request, @RequestParam Integer[] ids) {
        WebUser wu = RequestUtils.getWebUser(request);
        String subjectCode = "";
        for (Integer id : ids) {
            ExamStudent student = studentService.findById(id);
            subjectCode = student.getSubjectCode();
            if (student != null && subjectCheck(student.getSubjectCode(), wu)) {
                try {
                    if (lockService.trylock(LockType.STUDENT, student.getId()) && inspectedService.cancel(id)) {
                    }
                } catch (Exception e) {
                    log.error("back inspected error", e);
                } finally {
                    lockService.unlock(LockType.STUDENT, student.getId());
                }
            }
        }
        return "redirect:/admin/exam/inspected/list?subjectCode=" + subjectCode;
    }

    @Logging(menu = "成绩复核进度查询", type = LogType.QUERY)
    @RequestMapping(value = "/info")
    public String info(HttpServletRequest request, Model model, ExamSubjectSearchQuery query) {
        WebUser wu = RequestUtils.getWebUser(request);
        int examId = getSessionExamId(request);
        query.setExamId(examId);
        Set<String> unFinishSet = studentService.findSubjectUnInspectedByExamId(examId);
        if (query.getFinished() != null) {
            String subjectCodeIn = StringUtils.join(unFinishSet, ",");
            if (query.getFinished()) {
                query.setCodeNotIn(subjectCodeIn);
            } else {
                query.setCodeIn(subjectCodeIn);
            }
        }
        if (wu.isSubjectHeader()) {
            String subjectCodeIn = StringUtils.join(wu.getSubjectCodeSet(), ",");
            query.setCodeIn(subjectCodeIn);
        }
        query = subjectService.findByQuery(query);
        if (unFinishSet.isEmpty() && query.getFinished() != null && !query.getFinished()) {
            query.setResult(new ArrayList<ExamSubject>());
        }
        List<InspectedSubjectVO> list = new LinkedList<InspectedSubjectVO>();
        for (ExamSubject subject : query.getResult()) {
            InspectedSubjectVO vo = new InspectedSubjectVO();
            vo.setSubject(subject);
            ExamStudentSearchQuery sQuery = new ExamStudentSearchQuery();
            sQuery.setExamId(examId);
            sQuery.setUpload(true);
            sQuery.setAbsent(false);
            sQuery.setBreach(false);
            sQuery.setSubjectCode(subject.getCode());
            sQuery.addStatus(SubjectiveStatus.INSPECTED);
            long inspectedCount = studentService.countByQuery(sQuery);
            vo.setInspectedCount(inspectedCount);
            sQuery.addStatus(SubjectiveStatus.MARKED);
            long totalCount = studentService.countByQuery(sQuery);
            vo.setTotalCount(totalCount);
            vo.setLeftCount(totalCount - inspectedCount);
            String percent = totalCount > 0 ?
                    (new DecimalFormat("####.###").format(inspectedCount * 100.0 / totalCount) + "%") :
                    "100%";
            vo.setPercent(percent);
            vo.setFinishCount(studentService.findMinInspectCountByExamIdAndSubjectCode(examId, subject.getCode()));
            list.add(vo);
        }
        ExamStudentSearchQuery sQuery = new ExamStudentSearchQuery();
        sQuery.setExamId(examId);
        sQuery.setUpload(true);
        sQuery.setAbsent(false);
        sQuery.setBreach(false);
        sQuery.addStatus(SubjectiveStatus.INSPECTED);
        long markedCount = studentService.countByQuery(sQuery);
        sQuery.addStatus(SubjectiveStatus.MARKED);
        long total = studentService.countByQuery(sQuery);
        model.addAttribute("unMarkedCount", total - markedCount);
        model.addAttribute("markedCount", markedCount);
        long subjectCount = subjectService.count(examId);
        model.addAttribute("unFinishCount", unFinishSet.size());
        model.addAttribute("finishCount", subjectCount - unFinishSet.size());
        model.addAttribute("resultList", list);
        model.addAttribute("query", query);
        model.addAttribute("subjectList", getExamSubject(examId, wu));
        return "modules/exam/inspectedInfo";
    }
}
